/*
 *  (c) Copyright Enterprise Computing Research Group (ECRG),
 *  National University of Ireland, Galway 2003/2004.
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of
 *  the GNU Lesser General Public License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE and
 *  no warranty that the program does not infringe the Intellectual Property rights of a third party.
 *  See the GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License along with this program;
 *  if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
/**
 *  Interface point for JMS-MTP and JADE platform <p>
 *
 *  Handles the send and receving of JMS messages to and from the JADE platform
 *  </p>
 *
 * @author     Edward Curry - NUI, Galway
 * @version    1.0 14 January 2004
 */

package ie.nuigalway.ecrg.jade.jmsmtp;
import jade.core.Profile;

import jade.domain.FIPAAgentManagement.Envelope;

import jade.mtp.InChannel;
import jade.mtp.MTP;
import jade.mtp.MTPException;
import jade.mtp.TransportAddress;

import java.net.URL;
import java.net.URLClassLoader;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;

public abstract interface JMSMTP {

  /**
   *  Activate a default Address
   *
   * @param  disp           Passed messages to the platform
   * @param  p              Description of the Parameter
   * @return                Trnasport Address for the address activated
   * @throws  MTPException  Error during address activation
   */
  public TransportAddress activate(InChannel.Dispatcher disp, Profile p)
       throws MTPException;


  /**
   *  Activate a default Address
   *
   * @param  disp           Passed messages to the platform
   * @return                Trnasport Address for the address activated
   * @throws  MTPException  Error during address activation
   */
  public TransportAddress activate(InChannel.Dispatcher disp)
       throws MTPException;


  /**
   *  Activate a specific TransportAddress
   *
   * @param  disp           Used to pass messages to the platform
   * @param  ta             Address to activate
   * @param  p              is the Profile from which the configuration
   *      parameters for this instance of JADE container can be retrieved
   * @throws  MTPException  Error during address activation
   */
  public void activate(InChannel.Dispatcher disp, TransportAddress ta, Profile p)
       throws MTPException;


  /**
   *  Activate a specific TransportAddress
   *
   * @param  disp           Used to pass messages to the platform
   * @param  ta             Address to activate
   * @throws  MTPException  Error during address activation
   */
  public void activate(InChannel.Dispatcher disp, TransportAddress ta)
       throws MTPException;


  /**
   *  Deactivate a specific address
   *
   * @param  ta             Address to deactivate
   * @throws  MTPException  Error during address deactivation
   */
  public void deactivate(TransportAddress ta) throws MTPException;


  /**
   *  Deactivate the MTP
   *
   * @throws  MTPException  Error with MTP deactivation
   */
  public void deactivate() throws MTPException;


  /**
   *  Deliver a message to a jmsTA
   *
   * @param  addr           Address to deliver too
   * @param  env            Envelope of message
   * @param  payload        Message payload
   * @throws  MTPException  Error during message send
   */
  public void deliver(String addr, Envelope env, byte[] payload)
       throws MTPException;


  /**
   *  Converts a string to a JMS Transport address
   *
   * @param  rep            Contains the address as a string
   * @return                TransportAddress The address as a JMS Transport
   *      Address
   * @throws  MTPException  Error in Address conversion
   */
  public TransportAddress strToAddr(String rep) throws MTPException;


  /**
   *  Given a TransportAddress convert it to a string
   *
   * @param  ta             The Address as a TransportAddress object
   * @return                String Address as a string
   * @throws  MTPException  Error in Address conversion
   */
  public String addrToStr(TransportAddress ta) throws MTPException;


  /**
   *  Return the name of this MTP
   *
   * @return    MTP Name
   */
  public String getName();


  /**
   *  Get prtocols supported by this MTP
   *
   * @return    The Protocols supported by this MTP
   */
  public String[] getSupportedProtocols();
}

